@@ -11,7 +11,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/libpq/be-secure.c,v 1.15.2.1 2002/11/07 18:47:08 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/libpq/be-secure.c,v 1.15.2.8 2003/01/08 23:34:37 momjian Exp $
  *
  *	  Since the server static private key ($DataDir/server.key)
  *	  will normally be stored unencrypted so that the database
@@ -116,7 +116,7 @@ void		secure_destroy(void);
 int			secure_open_server(Port *);
 void		secure_close(Port *);
 ssize_t		secure_read(Port *, void *ptr, size_t len);
-ssize_t		secure_write(Port *, const void *ptr, size_t len);
+ssize_t		secure_write(Port *, void *ptr, size_t len);
 
 #ifdef USE_SSL
 static DH  *load_dh_file(int keylength);
@@ -273,12 +273,6 @@ secure_read(Port *port, void *ptr, size_t len)
 #ifdef USE_SSL
 	if (port->ssl)
 	{
-		if (port->count > RENEGOTIATION_LIMIT)
-		{
-			SSL_renegotiate(port->ssl);
-			port->count = 0;
-		}
-
 		n = SSL_read(port->ssl, ptr, len);
 		switch (SSL_get_error(port->ssl, n))
 		{
@@ -286,12 +280,14 @@ secure_read(Port *port, void *ptr, size_t len)
 				port->count += n;
 				break;
 			case SSL_ERROR_WANT_READ:
+				n = secure_read(port, ptr, len);
 				break;
 			case SSL_ERROR_SYSCALL:
-				elog(ERROR, "SSL SYSCALL error: %s", strerror(errno));
+				if (n == -1)
+					elog(COMMERROR, "SSL SYSCALL error: %s", strerror(errno));
 				break;
 			case SSL_ERROR_SSL:
-				elog(ERROR, "SSL error: %s", SSLerrmessage());
+				elog(COMMERROR, "SSL error: %s", SSLerrmessage());
 				/* fall through */
 			case SSL_ERROR_ZERO_RETURN:
 				secure_close(port);
@@ -311,7 +307,7 @@ secure_read(Port *port, void *ptr, size_t len)
  *	Write data to a secure connection.
  */
 ssize_t
-secure_write(Port *port, const void *ptr, size_t len)
+secure_write(Port *port, void *ptr, size_t len)
 {
 	ssize_t		n;
 
@@ -324,7 +320,15 @@ secure_write(Port *port, const void *ptr, size_t len)
 	{
 		if (port->count > RENEGOTIATION_LIMIT)
 		{
-			SSL_renegotiate(port->ssl);
+		        SSL_set_session_id_context(port->ssl, (void *)&SSL_context, sizeof(SSL_context));
+
+		        if (SSL_renegotiate(port->ssl) <= 0)
+			  elog(COMMERROR, "SSL renegotiation failure");
+			if (SSL_do_handshake(port->ssl) <= 0)
+			  elog(COMMERROR, "SSL renegotiation failure");
+			port->ssl->state=SSL_ST_ACCEPT;
+			if (SSL_do_handshake(port->ssl) <= 0)
+			  elog(COMMERROR, "SSL renegotiation failure");
 			port->count = 0;
 		}
 
@@ -335,12 +339,14 @@ secure_write(Port *port, const void *ptr, size_t len)
 				port->count += n;
 				break;
 			case SSL_ERROR_WANT_WRITE:
+				n = secure_read(port, ptr, len);
 				break;
 			case SSL_ERROR_SYSCALL:
-				elog(ERROR, "SSL SYSCALL error: %s", strerror(errno));
+				if (n == -1)
+					elog(COMMERROR, "SSL SYSCALL error: %s", strerror(errno));
 				break;
 			case SSL_ERROR_SSL:
-				elog(ERROR, "SSL error: %s", SSLerrmessage());
+				elog(COMMERROR, "SSL error: %s", SSLerrmessage());
 				/* fall through */
 			case SSL_ERROR_ZERO_RETURN:
 				secure_close(port);
@@ -585,7 +591,7 @@ initialize_SSL(void)
 	{
 		SSL_library_init();
 		SSL_load_error_strings();
-		SSL_context = SSL_CTX_new(TLSv1_method());
+		SSL_context = SSL_CTX_new(SSLv23_method());
 		if (!SSL_context)
 		{
 			postmaster_error("failed to create SSL context: %s",
@@ -614,7 +620,9 @@ initialize_SSL(void)
 		if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0077) ||
 			buf.st_uid != getuid())
 		{
-			postmaster_error("bad permissions on private key file (%s)", fnbuf);
+			postmaster_error("bad permissions on private key file (%s)\n"
+"File must be owned by the proper user and must have no permissions for\n"
+"\"group\" or \"other\".", fnbuf);
 			ExitPostmaster(1);
 		}
 		if (!SSL_CTX_use_PrivateKey_file(SSL_context, fnbuf, SSL_FILETYPE_PEM))
@@ -633,7 +641,7 @@ initialize_SSL(void)
 
 	/* set up empheral DH keys */
 	SSL_CTX_set_tmp_dh_callback(SSL_context, tmp_dh_cb);
-	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_DH_USE);
+	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_DH_USE | SSL_OP_NO_SSLv2);
 
 	/* accept client certificates, but don't require them. */
 	snprintf(fnbuf, sizeof fnbuf, "%s/root.crt", DataDir);
@@ -676,7 +684,7 @@ open_server_SSL(Port *port)
 		!SSL_set_fd(port->ssl, port->sock) ||
 		SSL_accept(port->ssl) <= 0)
 	{
-		elog(ERROR, "failed to initialize SSL connection: %s", SSLerrmessage());
+		elog(COMMERROR, "failed to initialize SSL connection: %s", SSLerrmessage());
 		close_SSL(port);
 		return -1;
 	}
